#include "FileListConsts.au3"
#include "FileListFuncs.au3"

FileListSystemFolder( "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}", 1, False ) ; My Computer
;FileListSystemFolder( "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}\::{21EC2020-3AEA-1069-A2DD-08002B30309D}", 0 ) ; Control Panel
CloseDlls()
Exit

Func FileListSystemFolder( $sFullPath, $maxLevels = 1, $fFiles = True )
  ; $maxLevels is the maximum number of recursion levels
  ; Set $maxLevels = 0 to go through all levels
  ; Set $fFiles = False to list directories only
  Local $pidl, $pidlRel, $pIShellParent, $IShellParent
  ; Get the fully qualified PIDL associated with the folder
  $pidl = ILCreateFromPath( $sFullPath )
  ; Get an IShellFolder interface pointer ($pIShellParent) for the parent folder
  ; and a PIDL ($pidlRel) associated with the folder relative to the parent folder.
  SHBindToParent( $pidl, DllStructGetPtr( $tRIID_IShellFolder ), $pIShellParent, $pidlRel )
  ; Get the IShellFolder interface for the parent folder
  $IShellParent = ObjCreateInterface( $pIShellParent, $sIID_IShellFolder, $dtagIShellFolder )
  ; List the files in the folder $pidlRel relative to the parent folder with the IShellFolder interface $IShellParent
  FileListSystemFolderNext( $IShellParent, $pidlRel, $maxLevels, 1, $fFiles )
  ; Free the memory allocated by the PIDL
  CoTaskMemFree( $pidl )
EndFunc

; List the files in the folder $pidlRel relative to the parent folder with the IShellFolder interface $IShellParent
Func FileListSystemFolderNext( $IShellParent, $pidlRel, $maxLevels, $level, $fFiles )
  ; $maxLevels is the maximum number of recursion levels
  Local $pIShellFolder, $IShellFolder, $pIEnumIDList, $IEnumIDList, $iFetched
  Local $sIndent, $tSTRRET = DllStructCreate( $tagSTRRET ), $sParsing, $sName

  ; Indentation
  For $i = 0 To $level - 2
    $sIndent = $sIndent & "    "
  Next

  ; Parsing name and normal name of the parent folder
  $IShellParent.GetDisplayNameOf( $pidlRel, $SHGDN_FORPARSING, $tSTRRET )
  StrRetToBuf( DllStructGetPtr( $tSTRRET ), $NULL, $sParsing )
  ; Exceptions
  Switch $sParsing
    Case "{BA126ADB-2166-11D1-B1D0-00805FC1270E}"
      ; Control Panel, Local Area Connection
      CoTaskMemFree( $pidlRel )
      $IShellParent = 0
      Return
    Case "{BA126AE2-2166-11D1-B1D0-00805FC1270E}"
      ; Control Panel, Internet Connection
      CoTaskMemFree( $pidlRel )
      $IShellParent = 0
      Return
  EndSwitch
  $IShellParent.GetDisplayNameOf( $pidlRel, $SHGDN_NORMAL, $tSTRRET )
  StrRetToBuf( DllStructGetPtr( $tSTRRET ), $NULL, $sName )
  ConsoleWrite( $sIndent & $sName & "\" & @CRLF )
  If StringRight( $sParsing, 1 ) = "\" Then _
    $sParsing = StringLeft( $sParsing, StringLen( $sParsing ) - 1 )
  ConsoleWrite( $sIndent & $sParsing & "\" & @CRLF )

  If $maxLevels And $level > $maxLevels Then
    CoTaskMemFree( $pidlRel )
    $IShellParent = 0
    Return
  EndIf

  ; Get an IShellFolder interface for the folder represented by $pidlRel
  $IShellParent.BindToObject( $pidlRel, $NULL, $tRIID_IShellFolder, $pIShellFolder )
  $IShellFolder = ObjCreateInterface( $pIShellFolder, $sIID_IShellFolder, $dtagIShellFolder )
  CoTaskMemFree( $pidlRel )
  $IShellParent = 0

  If $fFiles Then

    ; --- Print the name of the files ---
    ; (Files in the folder represented by the $pidlRel parameter)

    ; Get an IEnumIDList interface pointer ($pIEnumIDList) for the enumeration object of the folder
    If $IShellFolder.EnumObjects( $NULL, BitOR( $SHCONTF_NONFOLDERS, $SHCONTF_INCLUDEHIDDEN ), $pIEnumIDList ) = $S_OK Then
      ; Get the IEnumIDList interface
      $IEnumIDList = ObjCreateInterface( $pIEnumIDList, $sIID_IEnumIDList, $dtagIEnumIDList )
      ; Enumerate the files
      $sIndent = $sIndent & "    "
      While $IEnumIDList.Next( 1, $pidlRel, $iFetched ) = $S_OK
        $IShellFolder.GetDisplayNameOf( $pidlRel, $SHGDN_NORMAL, $tSTRRET )
        StrRetToBuf( DllStructGetPtr( $tSTRRET ), $NULL, $sName )
        ConsoleWrite( $sIndent & $sName & @CRLF )
        CoTaskMemFree( $pidlRel )
      WEnd
      $IEnumIDList = 0
    EndIf
  EndIf

  ; --- List the files of the folders by recursion ---
  ; (Folders in the folder represented by the $pidlRel parameter)

  ; Get an IEnumIDList interface pointer ($pIEnumIDList) for the enumeration object of the folder
  If $IShellFolder.EnumObjects( $NULL, BitOR( $SHCONTF_FOLDERS, $SHCONTF_INCLUDEHIDDEN ), $pIEnumIDList ) = $S_OK Then
    ; Get the IEnumIDList interface
    $IEnumIDList = ObjCreateInterface( $pIEnumIDList, $sIID_IEnumIDList, $dtagIEnumIDList )
    ; Enumerate the folders
    While $IEnumIDList.Next( 1, $pidlRel, $iFetched ) = $S_OK
      FileListSystemFolderNext( $IShellFolder, $pidlRel, $maxLevels, $level + 1, $fFiles )
    WEnd
    $IEnumIDList = 0
  EndIf
EndFunc
